part of calendar;

/// Callback to notify the controller properties changes
typedef CalendarValueChangedCallback = void Function(String);

/// Notifier to notify the controller properties changes
class CalendarValueChangedNotifier {
  List<CalendarValueChangedCallback> _listeners;

  //// Add the listener used to call back when calendar value changed.
  void addPropertyChangedListener(CalendarValueChangedCallback _listener) {
    _listeners ??= <CalendarValueChangedCallback>[];
    _listeners.add(_listener);
  }

  //// remove the listener used for notify the calendar value changes.
  void removePropertyChangedListener(CalendarValueChangedCallback _listener) {
    if (_listeners == null) {
      return;
    }

    _listeners.remove(_listener);
  }

  //// Notify to the listener while calendar value changed.
  void notifyPropertyChangedListeners(String property) {
    if (_listeners == null) {
      return;
    }

    for (CalendarValueChangedCallback listener in _listeners) {
      if (listener != null) {
        listener(property);
      }
    }
  }

  @mustCallSuper
  void dispose() {
    _listeners = null;
  }
}

/// Controller used for programmatic date navigation and date selection in calendar widget
///
/// ```dart
/// class MyAppState extends State<MyApp>{
///
///  CalendarController _calendarController;
///  @override
///  initState(){
///    _calendarController = CalendarController();
///    _calendarController.selectedDate = DateTime(2022, 02, 05);
///    _calendarController.displayDate = DateTime(2022, 02, 05);
///    super.initState();
///  }
///
///  @override
///  Widget build(BuildContext context) {
///    return MaterialApp(
///      home: Scaffold(
///        body: SfCalendar(
///          view: CalendarView.month,
///          controller: _calendarController,
///        ),
///      ),
///    );
///  }
///}
/// ```
class CalendarController extends CalendarValueChangedNotifier {
  DateTime _selectedDate;
  DateTime _displayDate;

  DateTime get selectedDate => _selectedDate;

  /// selectedDate used to select a date programmatically on calendar
  ///
  /// ```dart
  /// class MyAppState extends State<MyApp>{
  ///
  ///  CalendarController _calendarController;
  ///  @override
  ///  initState(){
  ///    _calendarController = CalendarController();
  ///    _calendarController.selectedDate = DateTime(2022, 02, 05);
  ///    _calendarController.displayDate = DateTime(2022, 02, 05);
  ///    super.initState();
  ///  }
  ///
  ///  @override
  ///  Widget build(BuildContext context) {
  ///    return MaterialApp(
  ///      home: Scaffold(
  ///        body: SfCalendar(
  ///          view: CalendarView.month,
  ///          controller: _calendarController,
  ///        ),
  ///      ),
  ///    );
  ///  }
  ///}
  /// ```
  set selectedDate(DateTime date) {
    if (isSameDate(_selectedDate, date) &&
        _isSameTimeSlot(_selectedDate, date)) {
      return;
    }

    _selectedDate = date;
    notifyPropertyChangedListeners('selectedDate');
  }

  DateTime get displayDate => _displayDate;

  /// displayDate used to navigate to a date programmatically on calendar
  ///
  /// ```dart
  /// class MyAppState extends State<MyApp>{
  ///
  /// CalendarController _calendarController;
  /// @override
  ///  initState(){
  ///    _calendarController = CalendarController();
  ///    _calendarController.selectedDate = DateTime(2022, 02, 05);
  ///    _calendarController.displayDate = DateTime(2022, 02, 05);
  ///    super.initState();
  ///  }
  ///
  ///  @override
  ///  Widget build(BuildContext context) {
  ///    return MaterialApp(
  ///      home: Scaffold(
  ///        body: SfCalendar(
  ///          view: CalendarView.month,
  ///          controller: _calendarController,
  ///        ),
  ///      ),
  ///    );
  ///  }
  ///}
  /// ```
  set displayDate(DateTime date) {
    if (isSameDate(_displayDate, date)) {
      return;
    }

    _displayDate = date;
    notifyPropertyChangedListeners('displayDate');
  }

  /// forward used to programmatically navigate to next visible date
  ///
  /// ```dart
  /// class MyAppState extends State<MyApp> {
  ///  CalendarController _calendarController;
  ///
  ///  @override
  ///  initState() {
  ///    _calendarController = CalendarController();
  ///    _calendarController.selectedDate = DateTime(2022, 02, 05);
  ///    _calendarController.displayDate = DateTime(2022, 02, 05);
  ///    super.initState();
  ///  }
  ///
  ///  @override
  ///  Widget build(BuildContext context) {
  ///    return MaterialApp(
  ///      home: Scaffold(
  ///        appBar: AppBar(
  ///          title: Text('Calendar Demo'),
  ///          actions: <Widget>[
  ///            IconButton(
  ///              icon: Icon(Icons.arrow_forward),
  ///              onPressed: () {
  ///                _calendarController.forward();
  ///              },
  ///            ),
  ///          ],
  ///          leading: IconButton(
  ///            icon: Icon(Icons.arrow_back),
  ///            onPressed: () {
  ///              _calendarController.backward();
  ///            },
  ///          ),
  ///        ),
  ///        body: SfCalendar(
  ///          view: CalendarView.month,
  ///          controller: _calendarController,
  ///        ),
  ///      ),
  ///    );
  ///  }
  ///}
  /// ```
  VoidCallback _forward;

  void forward() {
    if (_forward == null) {
      return;
    }

    _forward();
  }

  /// backward used to programmatically navigate to previous visible date
  ///
  /// ```dart
  /// class MyAppState extends State<MyApp> {
  ///  CalendarController _calendarController;
  ///
  ///  @override
  ///  initState() {
  ///    _calendarController = CalendarController();
  ///    _calendarController.selectedDate = DateTime(2022, 02, 05);
  ///    _calendarController.displayDate = DateTime(2022, 02, 05);
  ///    super.initState();
  ///  }
  ///
  ///  @override
  ///  Widget build(BuildContext context) {
  ///    return MaterialApp(
  ///      home: Scaffold(
  ///        appBar: AppBar(
  ///          title: Text('Calendar Demo'),
  ///          actions: <Widget>[
  ///            IconButton(
  ///              icon: Icon(Icons.arrow_forward),
  ///              onPressed: () {
  ///                _calendarController.forward();
  ///              },
  ///            ),
  ///          ],
  ///          leading: IconButton(
  ///            icon: Icon(Icons.arrow_back),
  ///            onPressed: () {
  ///              _calendarController.backward();
  ///            },
  ///          ),
  ///        ),
  ///        body: SfCalendar(
  ///          view: CalendarView.month,
  ///          controller: _calendarController,
  ///        ),
  ///      ),
  ///    );
  ///  }
  ///}
  /// ```
  VoidCallback _backward;

  void backward() {
    if (_backward == null) {
      return;
    }

    _backward();
  }
}
